@@ -20,16 +20,22 @@ from fandak.core.evaluators import Evaluator, GeneralEvaluatorResult
 from fandak.core.models import Model, GeneralLoss, GeneralForwardOut
 from fandak.utils.metrics import ScalarMetricCollection
 from fandak.utils.misc import get_git_commit_hash, print_with_time
+from fandak.utils.locks import lock
+
+RUN_INFO_TEMPLATE = """Time: {time}  
+Command: {command}  
+Git hash: {hash}  
 
-RUN_INFO_TEMPLATE = """Time: {time}
-Command: {command}
-Git hash: {hash}
 -----------------------------------------
+
 {config}
+
 """
 
 RUN_EXTRA_TEMPLATE = """-----------------------------------------
+
 {extra}
+
 """
 
 TORCH_EXT = "trc"
@@ -98,8 +104,13 @@ class Trainer(ABC):
     def _save_info_of_run(self):
         extra_info = self.extra_info_for_run()
 
+        # fixme: assuming self.cfg has a `.dump` function.
         config_dump = self.cfg.dump()
 
+        # adding 4 spaces at the beginning of each line
+        # this is because now tensorboard's text section can visualize it as code block
+        config_dump = "\n".join([f"    {l}" for l in config_dump.split("\n")])
+
         final_value = RUN_INFO_TEMPLATE
 
         result = final_value.format(
@@ -112,22 +123,15 @@ class Trainer(ABC):
         if extra_info:
             result += RUN_EXTRA_TEMPLATE.format(extra=extra_info)
 
-        with open(self.run_folder / Path("info.txt"), "w") as f:
+        with open(self.run_folder / Path("info.md"), "w") as f:
             f.write(result)
 
-        # fixme: there is still a lot of problems with viewing here
-        result_markdown_linebreak_fixed = result.replace("\n", "  \n")
-
         # Add the info to tensorboard for easier observation.
-        self.writer.add_text("info.txt", result_markdown_linebreak_fixed)
+        self.writer.add_text("info", result)
 
-        # also optionally save the config file.
-        # is the config object has a `.dump()` function, we will call it and save
-        # its content in a `config.yaml` format.
-        if hasattr(self.cfg, "dump") and callable(self.cfg.dump):
-            config_path = self.run_folder / "config.yaml"
-            with open(config_path, "w") as f:
-                f.write(self.cfg.dump())
+        config_path = self.run_folder / "config.yaml"
+        with open(config_path, "w") as f:
+            f.write(self.cfg.dump())
 
     def save_run_report(
         self, report: str, name: str = "report", extension: str = "txt"
@@ -332,14 +336,15 @@ class Trainer(ABC):
     def _figure_run_number(self) -> int:
         # fixme: this is not thread safe!
         max_run = 0
-        for f in self.experiment_folder.iterdir():
-            if f.is_dir():
-                try:
-                    f = int(str(f.name))
-                except ValueError:
-                    continue
-                if f > max_run:
-                    max_run = f
+        with lock(f"run_number.{str(self.experiment_folder)}"):
+            for f in self.experiment_folder.iterdir():
+                if f.is_dir():
+                    try:
+                        f = int(str(f.name))
+                    except ValueError:
+                        continue
+                    if f > max_run:
+                        max_run = f
 
         return max_run + 1
 
